/* $Id: sfskeymisc.C,v 1.1 1999/04/23 05:30:35 kaminsky Exp $ */

/*
 *
 * Copyright (C) 1999 Michael Kaminsky (kaminsky@lcs.mit.edu)
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */


#include "sfskey.h"

static void
sfs_gethostidcon_cb (sfs_connectres *resp, callback<void, str>::ref cb, 
		     clnt_stat err)
{
  sfs_hash authhostid;

  if (err) {
    warn << err << "\n";
    cb (str (NULL));
  }
  else if (resp->status) {
    warn << resp->status << "\n";
    cb (str (NULL));
  }
  else if (!sfs_mkhostid (&authhostid, resp->reply->host)) {
    warn << "can't create hostid from server-returned hostinfo\n";
    cb (str (NULL));
  }
  else {
    cb (resp->reply->host.hostname << ":" 
	<< armor32 (str (authhostid.base (), sizeof (sfs_hash))));
  }

  delete resp;
}

void
sfs_gethostidcon (str hostname, sfs_service service, 
		  callback<void, str>::ref cb, int fd)
{
  if (fd < 0) {
    fatal << "Couldn't not connect to server: "
	  << strerror (errno) << "\n";
  }

  ref<aclnt> c = aclnt::alloc (axprt_stream::alloc (fd),
			       sfs_program_1);

  if (!hostname) {
    hostname = myname ();
    if (!hostname)
      fatal << "Could not determine our own hostname.\n";
  }

  sfs_connectarg arg;
  arg.release = SFS_RELEASE;
  arg.service = service;
  arg.name = hostname;
  //  arg.hostid = NULL;

  sfs_connectres *resp = New sfs_connectres;
  c->call (SFSPROC_CONNECT, &arg, resp, 
	   wrap (sfs_gethostidcon_cb, resp, cb));
}

void
sfs_gethostid (str hostname, sfs_service service, 
	       callback<void, str>::ref cb)
{
  if (hostname) {
    tcpconnect (hostname, sfs_port, 
		wrap (sfs_gethostidcon, hostname, service, cb));
  }
  else {
    if (service != SFS_AUTHSERV)
      fatal << "Only authserv (not sfsrwsd) support unix-domain sockets.\n";
    int fd = suidgetfd ("authserv");
    if (fd < 0)
      fatal << "Could not find 'authserv' unix domain socket.\n";
    sfs_gethostidcon (hostname, service, cb, fd);
  }
}

bool
parse_userhost (const str &raw, str *user, str *host)
{
  rxx r (rxx ("^([a-z0-9]+)?@([A-Za-z0-9\\.\\-]+)$"));
  if (!r.match (raw))
    return false;

  if (r[1]) {
    *user = r[1];
  }
  else {
    if (!(*user = getlogin ()))
      return false;
  }

  *host = r[2];

  return true;
}
