// -*-c++-*-
/* $Id: sfscd.h,v 1.35 1999/03/25 01:24:20 dm Exp $ */

/*
 *
 * Copyright (C) 1998, 1999 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "arpc.h"
#include "afsnode.h"
#include "nfsmounter.h"
#include "sfscd_prot.h"
#include "sfsmisc.h"
#include "crypt.h"
#include "rabin.h"
#include "qhash.h"

#ifdef HAVE_NFS_V3
# define V3FLAG NMOPT_NFS3
#else
# define V3FLAG 0
#endif


#include "afsroot.h"

extern bool terminating;
extern bool nomounting;

extern ref<afsroot> afs_root;	// Files everyone sees in root directory
extern ref<afsdir> afs_sfsroot;	// Files seen by root during mount syscalls
extern ref<afsdir> afs_naroot;	// Files seen by users with no agents

extern ref<afsdir> afs_wait;	// Places people get "stuck" in readlink...
extern ref<afsdir> afs_mnt;	// ...to avoid locking the root directory

extern int afsfd;

void afs_init (cbv);

inline nfsmnt_handle
nfs_fh2tobytes (const nfs_fh &fh)
{
  nfsmnt_handle res;
  res.set (fh.data);
  return res;
}

void mnt_init ();
void mnt_mount (int s, str hostname, str path,
		int flags, const nfsmnt_handle &fh,
		callback<void, int, u_int64_t>::ref cb);
void mnt_remount (str path, int flags, cbi cb);
void mnt_umount (str path, int flags, cbi cb);
void mnt_umountall (int flags, cbi cb);

struct usrinfo {
  const sfs_aid aid;

  ref<afsusrdir> root;
  ihash_entry<usrinfo> hlink;
  ptr<asrv> sh;
  ptr<aclnt> ch;

  usrinfo (sfs_aid);
  ~usrinfo ();
  void dispatch (svccb *);
  static usrinfo *lookup (const authunix_parms *);

  void authreq (sfscd_authreq *ar, svccb *sbp);
};
extern ihash<const sfs_aid, usrinfo, &usrinfo::aid, &usrinfo::hlink> usrtab;

struct cdaemon;

class srvinfo {
  typedef callback<void, srvinfo *, int>::ref alloccb_t;

  timecb_t *tmo;
  vector<alloccb_t> waitq;
  sfs_connectres conres;
  sfscd_mountres mntres;
  bool waiting;
  bool destroyed;
  bool cdmounted;
  bool visible_flag;

  srvinfo (const str &pathname, const str &hostname, const sfs_hash &hostid);
  PRIVDEST ~srvinfo ();

  void timeout (bool start = true);
  void fail (int);
  void connected (int);
  void gotconres (int, clnt_stat);
  void gotmntres (clnt_stat);
  void gotnfsmntres (int, u_int64_t);
  void ready ();
  void unmountcb (cbi::ptr, int);
  static void printdev (strbuf *sb, srvinfo *si);

public:
  u_int64_t devno;
  int error;

  const str path;
  const str hostname;
  const sfs_hash hostid;
  cdaemon *cdp;

  ihash_entry<srvinfo> hlink;
  list_entry<srvinfo> cdlink;

  void unmount (int flags, cbi::ptr cb = NULL);
  void destroy (bool stale);
  static srvinfo *lookup (const str &path);
  static void alloc (const str &path, alloccb_t cb);
  static void destroy (const str &path, cdaemon *cdp, bool stale);
  static void show (const str &path, cdaemon *cdp, bool showit);
  static int pathstate (const str &path);
  static str devlist ();
};

class cdaemon {
  void eof ();
  void dispatch (svccb *sbp);

public:
  str name;
  const int32_t prog;
  vector<int32_t> vers;
  vector<str> argv;
  ptr<axprt_unix> x;
  ptr<aclnt> c;
  ptr<asrv> s;
  list<srvinfo, &srvinfo::cdlink> servers;
  ihash_entry<cdaemon> hlink;

  cdaemon (int32_t);
  ~cdaemon ();

  bool launch (bool synchronous);
  static cdaemon *lookup (int32_t prog, int32_t vers);
};
extern ihash<const int32_t, cdaemon, &cdaemon::prog, &cdaemon::hlink> cdtab;

void usrinfo_init ();
bool isnfsmounter (const authunix_parms *aup);
void flushpath (str path);
