/* $Id: sfsrwcd.C,v 1.23 1999/03/24 07:25:31 dm Exp $ */

/*
 *
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "sfsrwcd.h"

const int pubkeybits = 1024;
ptr<rabin_priv> client_key;

ptr<axprt_unix> cdx;
ptr<aclnt> cdc;
static ptr<asrv> cds;

auto_auth qhash_lookup_return<auto_auth>::nullauth;

static void
client_keygen ()
{
  client_key = New refcounted<rabin_priv> (rabin_keygen (pubkeybits));
  timecb (time (NULL) + 3600, wrap (client_keygen));
}

static void
cd_dispatch (struct svccb *sbp)
{
  static bool init = false;

  if (!sbp)
    fatal ("EOF from sfscd\n");
  switch (sbp->proc ()) {
  case SFSCDPROC_NULL:
    sbp->reply (NULL);
    break;
  case SFSCDPROC_INIT:
    {
      if (init)
	fatal ("SFSCDPROC_INIT called twice\n");
      init = true;
      sfscd_initarg *iap = sbp->template getarg<sfscd_initarg> ();
      sfs_suidserv (iap->name, wrap (sfsctl_accept));
      sbp->reply (NULL);
      break;
    }
  case SFSCDPROC_MOUNT:
    server::alloc (sbp, cdx->recvfd ());
    break;
  case SFSCDPROC_UNMOUNT:
    if (server *s = server::lookup (*sbp->template getarg<nfspath3> ()))
      s->destroy ();
    sbp->reply (NULL);
    break;
  case SFSCDPROC_FLUSHAUTH:
    {
      sfs_aid aid = *sbp->template getarg<sfs_aid> ();
      server::flushuser (aid);
      sbp->reply (NULL);
      break;
    }
  default:
    sbp->reject (PROC_UNAVAIL);
    break;
  }
}

int
main (int argc, char **argv)
{
  setprogname (argv[0]);
  warn ("pid %d\n", getpid ());

  if (argc != 1)
    fatal ("usage: %s\n", progname.cstr ());

  sfsconst_init ();
  if (!builddir)
    random_set_seedfile (strbuf ("%s/random_seed", sfsdir.cstr ()));
  random_init ();

  char fhkeydat[53];
  rnd.getbytes (fhkeydat, sizeof (fhkeydat));
  fhkey.setkey (fhkeydat, sizeof (fhkeydat));
  bzero (fhkeydat, sizeof (fhkeydat));

  client_keygen ();
  server::init ();

  if (!(cdx = axprt_unix_stdin ())
      || !(cds = asrv::alloc (cdx, sfscd_program_1, wrap (cd_dispatch)))
      || !(cdc = aclnt::alloc (cdx, sfscdcb_program_1)))
    exit (1);

  amain ();
}
